#!/usr/bin/python

""" The whole point of input specifications is that they'll contain more
information than is available elsewhere. This begs the question of what kind of
process should be used to annotate the existing resources, in this case the text
file containing XPath locators for each significant node in each of the UBL
document types.

The XPath data files look something like this:

1   1..1 /rs:OrderResponseSimple/
2   1..1 /rs:OrderResponseSimple/rs:ID
2.1 0..1 /rs:OrderResponseSimple/rs:ID/@identificationSchemeAgencyID

Separately, in another file annot.txt I gather a listing of "leaves", i.e. for
the above, it would be:

rs:ID stringinput
@identificationSchemeAgencyID hidden
etc.

The 2nd value is the 'data processing collection intent' for all the places in
the XML that use that exact leaf name.

The list of possible data collection intents is as follows:
    * hidden (not presented to the end user)
    * group (recursive)
    * stringinput
    * numberinput (whole numbers)
    * decimalinput (decimal numbers)
    * dateinput
    * datetimeinput
    * textarea
    * select
    * select1
    * range

Processing combines these two lists into a single specification.

If you look at annot.txt in depth, it seems that the uniform naming of leaves
makes it such that a set of rules (if the leafname contains '@'->hidden, and if
it contains "Amount"->decimalnumber, etc.) could express the same information in
a format more flexible in the face of change.

For now, the order in the XPath file is taken as the recommended order for
navigation. This might even be correct.

Next steps:

TODO: intelligently deal with cardinality, so that 1..7 for example gets treated
as a repeating section

TODO: Better handle recursion.

TODO: Identify places that actually use lists, then list possible values

"""

import sys

class blank: pass
ubldata = blank()
ubldata.doctype = {}
ubldata.annot = {}
ubldata.all_leaves = {}

path = "xpaths/text/"

files = [ "DespatchAdvice", "Invoice", "Order", "OrderCancellation",
          "OrderChange", "OrderResponse", "OrderResponseSimple", "ReceiptAdvice" ]

#files = [ "OrderResponseSimple" ]

suffix = "-XPath.txt"

def main():
    gather()
    report()

def gather():
    "read in all needed data into handy data structures"
    f = open("annot.txt") # our added data
    for line in f:
        if line[0] != "#":
            tokens = line.split()
            ubldata.annot[tokens[0]] = (tokens[1])
    f.close()
    al2 = {}
    for file in files:
        entries = []
        f = open(path + file + suffix)
        for line in f:
            # parse and get into a self-documenting data structure
            if line[0] != "#":
                tokens = line.split()
                record = blank()
                record.numref = tokens[0]
                record.cardinality = tokens[1]
                record.card_min = int(tokens[1].split("..")[0])
                try:
                    record.card_max = int(tokens[1].split("..")[1])
                except ValueError:
                    record.card_max = sys.maxint
                record.path = tokens[2]
                record.leaf = record.path.split('/')[-1]
                record.terminal = True
                if record.leaf == "":
                    record.leaf = record.path.split('/')[-2] + "/"
                    record.terminal = False
                if ubldata.all_leaves.has_key(record.leaf):
                    ubldata.all_leaves[record.leaf] = ubldata.all_leaves[record.leaf] + 1
                    al2[record.leaf] = al2[record.leaf] +1
                else:
                    ubldata.all_leaves[record.leaf] = 1
                    al2[record.leaf] = 1
                record.root = record.path.split('/')[1]
                record.root2 = record.path.split('/')[2]
                entries.append(record)
        ubldata.doctype[file] = entries
        f.close()

    # save the list of leaves. Rename this to annot.txt, and add appropriate
    # annotations to each line, then the following code will use that info
    f = open("leaves.txt", "w")
    for leaf in ubldata.all_leaves:
        f.write(leaf)
        f.write("\n")
    f.close()

def report():
    "write out the processed data"
    # any overall reporting goes here


    # file-specific reports
    for file in files:
        print
        print "Report for " + file
        print "============================"
        rep_summary(file)

def rep_summary(file):
    lastroot2 = ""
    print "Summary of " + file
    for record in ubldata.doctype[file]:
        if lastroot2 != record.root2:
            lastroot2 = record.root2
            reqstr = ""
            if record.card_min > 0: reqstr = " (required)"
            dt = ubldata.annot[record.leaf]
            if dt != "hidden":
                print record.cardinality + " " + record.leaf + " " + dt + reqstr

if __name__ == '__main__':
    main()

#end